@@ -1,3 +1,4 @@
+"""EchoStream Terraform module generator."""
 import os
 import re
 import subprocess
@@ -11,6 +12,7 @@ from gql import Client as GqlClient
 from gql import gql
 from gql.transport.requests import RequestsHTTPTransport
 from pycognito.utils import RequestsSrpAuth
+from termcolor import colored, cprint
 
 from .data_sources import factory as data_source_factory
 from .objects import (
@@ -22,22 +24,18 @@ from .objects import (
     TerraformObject,
     encode_terraform,
 )
-from .resources import factory as resource_factory, Resource
+from .resources import Resource
+from .resources import factory as resource_factory
 
 DEFAULT_APPSYNC_ENDPOINT = "https://api-prod.us-east-1.echo.stream/graphql"
 
 
 def main():
+    """Dispatch the command line interface."""
     dispatch_command(terrafy)
 
 
-"""
-@wrap_errors(
-    errors=[Exception], processor=lambda exc_info: f"\033[91m{exc_info}\033[00m"
-)
-"""
-
-
+@wrap_errors(processor=lambda err: colored(str(err), "red"))
 @arg(
     "--appsync-endpoint",
     help=f"The ApiUser's AppSync Endpoint. Defaults to {DEFAULT_APPSYNC_ENDPOINT}. Environment variable - ECHOSTREAM_APPSYNC_ENDPOINT",
@@ -80,6 +78,8 @@ def terrafy(
     All file creation is done in the current directory.
 
     Requires Terraform (>= 1.3.5) to be installed and available in the PATH."""
+    if os.name != "posix":
+        raise RuntimeError("echostream-terrafy only works on POSIX systems")
     appsync_endpoint = (
         appsync_endpoint
         or os.environ.get("ECHOSTREAM_APPSYNC_ENDPOINT")
@@ -118,7 +118,7 @@ def terrafy(
             else f"Terraform version must be >= 1.3.5, found {m.group(1)}"
         )
 
-    __print_cyan(f"Logging user {username} in to Tenant {tenant}")
+    cprint(f"Logging user {username} in to Tenant {tenant}", "cyan")
     gql_client = GqlClient(
         fetch_schema_from_transport=True,
         transport=RequestsHTTPTransport(
@@ -132,9 +132,9 @@ def terrafy(
             url=appsync_endpoint,
         ),
     )
-    __print_green("Login successful!")
+    cprint("Login successful!", "green")
 
-    __print_cyan(f"Terrafying EchoStream Tenant {tenant}")
+    cprint(f"Terrafying EchoStream Tenant {tenant}", "cyan")
 
     terraform_objects: list[TerraformObject] = list()
     for getter in (
@@ -148,18 +148,19 @@ def terrafy(
         __process_apps,
         __process_nodes_and_edges,
     ):
-        stdout.write("\033[93m.\033[00m")
+        stdout.write(colored(".", "light_grey"))
         stdout.flush()
         terraform_objects.extend(getter(gql_client, tenant))
     stdout.write("\n")
 
-    __print_cyan("Initializing terraform")
+    cprint("Initializing terraform", "cyan")
     subprocess.run([terraform, "init"], capture_output=True, check=True)
-    __print_green("Terraform initialized!")
+    cprint("Terraform initialized!", "green")
 
     if os.path.exists("terraform.tfstate"):
-        __print_yellow(
-            "WARNING: terraform.tfstate already exists, clearing all outputs and resources!"
+        cprint(
+            "WARNING: terraform.tfstate already exists, clearing all outputs and resources!",
+            "yellow",
         )
         with open("terraform.tfstate", "rt") as f:
             tfstate = json.load(f)
@@ -168,7 +169,7 @@ def terrafy(
         with open("terraform.tfstate", "wt") as f:
             json.dump(tfstate, f, indent=2)
 
-    __print_cyan("Importing EchoStream resources")
+    cprint("Importing EchoStream resources", "cyan")
     env = dict(
         os.environ,
         ECHOSTREAM_APPSYNC_ENDPOINT=appsync_endpoint,
@@ -191,36 +192,26 @@ def terrafy(
                 raise Exception(
                     f"Error importing {obj.address}\n{cpe.stdout}\n{cpe.stderr}"
                 )
-            stdout.write("\033[93m.\033[00m")
+            stdout.write(colored(".", "light_grey"))
             stdout.flush()
     stdout.write("\n")
-    __print_green("EchoStream resources imported!")
+    cprint("EchoStream resources imported!", "green")
     os.remove("terraform.tfstate.backup")
-    __print_cyan("Confirming that infrastructure matches configuration")
+    cprint("Confirming that infrastructure matches configuration", "cyan")
     plan_check = subprocess.run([terraform, "plan"], capture_output=True, check=True)
     if not re.search(r"No changes.", plan_check.stdout.decode(), flags=re.MULTILINE):
-        __print_yellow(
-            f"WARNING: Tenant {tenant} infrastructure does not match the configuration. Plan output below:"
+        cprint(
+            f"WARNING: Tenant {tenant} infrastructure does not match the configuration. Plan output below:",
+            "yellow",
         )
         stdout.write("\n")
         print(plan_check.stdout.decode())
 
-    __print_green(f"EchoStream Tenant {tenant} Terrafy'd!!!")
-
-
-def __print_cyan(value: str) -> None:
-    print(f"\033[96m{value}\033[00m")
-
-
-def __print_green(value: str) -> None:
-    print(f"\033[92m{value}\033[00m")
-
-
-def __print_yellow(value: str) -> None:
-    print(f"\033[93m{value}\033[00m")
+    cprint(f"EchoStream Tenant {tenant} Terrafy'd!!!", "green")
 
 
 def __process_api_users(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
+    """Process API Users"""
     query = gql(
         """
         query listApiUsers($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -262,6 +253,7 @@ def __process_api_users(gql_client: GqlClient, tenant: str) -> list[TerraformObj
 
 
 def __process_apps(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
+    """Process Apps"""
     query = gql(
         """
         query listApps($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -322,6 +314,7 @@ def __process_apps(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
 
 
 def __process_functions(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
+    """Process Functions"""
     query = gql(
         """
         query listFunctions($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -382,6 +375,7 @@ def __process_functions(gql_client: GqlClient, tenant: str) -> list[TerraformObj
 
 
 def __process_kms_keys(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
+    """Process KMS Keys"""
     query = gql(
         """
         query listKmsKeys($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -422,6 +416,7 @@ def __process_kms_keys(gql_client: GqlClient, tenant: str) -> list[TerraformObje
 
 
 def __process_main(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
+    """Process Main"""
     main_json = dict(
         terraform=dict(
             required_providers=dict(
@@ -443,6 +438,7 @@ def __process_main(gql_client: GqlClient, tenant: str) -> list[TerraformObject]:
 def __process_managed_node_types(
     gql_client: GqlClient, tenant: str
 ) -> list[TerraformObject]:
+    """Process Managed Node Types"""
     query = gql(
         """
         query listManagedNodeTypes($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -514,6 +510,7 @@ def __process_managed_node_types(
 def __process_message_types(
     gql_client: GqlClient, tenant: str
 ) -> list[TerraformObject]:
+    """Process Message Types"""
     query = gql(
         """
         query listMessageTypes($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -570,6 +567,7 @@ def __process_message_types(
 def __process_nodes_and_edges(
     gql_client: GqlClient, tenant: str
 ) -> list[TerraformObject]:
+    """Process Nodes and Edges"""
     query = gql(
         """
         query listNodes($tenant: String!, $exclusiveStartKey: AWSJSON) {
@@ -984,6 +982,7 @@ def __process_nodes_and_edges(
 def __process_tenant_and_tenant_users(
     gql_client: GqlClient, tenant: str
 ) -> list[TerraformObject]:
+    """Process tenant and tenant users."""
     query = gql(
         """
         query getTenant($tenant: String!) {
